import json

from .exceptions import JSONLoad


def _load_json(data: str):
    try:
        return json.loads(data)
    except Exception:
        err = "Unable to load JSON"
        raise JSONLoad(err) from None



class FingerprintPattern(Exception):
    """The fingerprint pattern is not a valid JSON fingerprint pattern."""

    pass


class FingerprintVersion(Exception):
    """The fingerprint version is not a supported JSON fingerprint version."""

    pass


class HashFunction(Exception):
    """The hash function is not a supported JSON fingerprint hash function."""

    pass


class InputDataType(Exception):
    """The input data type is not string."""

    pass


class JSONLoad(Exception):
    """The input data is not valid JSON."""

    pass



from ._jfpv1 import _create_jfpv1_fingerprint
from ._load_json import _load_json
from ._validators import (
    _validate_hash_function,
    _validate_input_type,
    _validate_version,
)


def create(input: str, hash_function: str, version: int) -> str:
    """Create JSON fingerprints with the selected hash function and JSON fingerprint algorithm version.

    Args:
        input (str):
            JSON input in string format.
        hash_function (str):
            One of the supported hash function names in string format (options: "sha256", "sha384", or "sha512").
        version (int):
            An integer indicating the JSON fingerprint algorithm version to be used (options: 1).

    Returns:
        str: A pre-formatted JSON fingerprint (example: "jfpv1${hash_function_name}${hash_hex_digest}").
    """
    _validate_version(version=version)
    _validate_input_type(input=input)
    _validate_hash_function(hash_function=hash_function, version=version)
    loaded = _load_json(data=input)
    return _create_jfpv1_fingerprint(data=loaded, hash_function=hash_function)



from ._create import create
from ._decode import decode
from ._find_matches import find_matches
from ._match import match
from .exceptions import (
    FingerprintPattern,
    FingerprintVersion,
    HashFunction,
    InputDataType,
    JSONLoad,
)



import json
import unittest

from json_fingerprint import _validators, create, exceptions, hash_functions


class TestHashFunctions(unittest.TestCase):
    def test_hash_function_module_options(self):
        """Test the 'hash_functions' module's SHA-2 options.

        Verify that:
        - Each supported hash function string is available ("sha256", "sha384", and "sha512")
        - The options are all-lowercase (exact match)
        """
        self.assertEqual(hash_functions.SHA256, "sha256")
        self.assertEqual(hash_functions.SHA384, "sha384")
        self.assertEqual(hash_functions.SHA512, "sha512")


if __name__ == "__main__":
    unittest.main()



import json
import unittest

from json_fingerprint import _validators, create, exceptions, hash_functions


class TestValidators(unittest.TestCase):
    def test_json_fingerprint_version(self):
        """Test JSON fingerprint FingerprintVersion exception.

        Verify that:
        - FingerprintVersion exception is properly raised with an unsupported version
        - FingerprintVersion exception is not raised with a supported version
        """
        with self.assertRaises(exceptions.FingerprintVersion):
            create(input=json.dumps({"foo": "bar"}), hash_function=hash_functions.SHA256, version=-1)

        _validators._validate_version(version=1)

        with self.assertRaises(exceptions.FingerprintVersion):
            _validators._validate_version(version=-1)

    def test_input_data_type(self):
        """Test jfpv1 input data type error.

        Verify that:
        - DataType exception is not raised with a valid string input
        - DataType exception is properly raised with a non-string input
        """
        with self.assertRaises(exceptions.InputDataType):
            create(input=123, hash_function=hash_functions.SHA256, version=1)

        try:
            _validators._validate_input_type(input="abc")
        except Exception as exc:
            err = "_validate_input_type() failed with a valid string"
            self.fail(f"{err}: {exc}")

        with self.assertRaises(exceptions.InputDataType):
            _validators._validate_input_type(input=123)

    def test_jfpv1_hash_function(self):
        """Test JSON fingerprint HashFunction exception.

        Verify that:
        - HashFunction exception is properly raised with an unsupported hash function selector
        - HashFunction exception is not raised with a supported hash function selector
        """
        with self.assertRaises(exceptions.HashFunction):
            create(input=json.dumps({"foo": "bar"}), hash_function="not123", version=1)

        try:
            _validators._validate_hash_function(hash_function=hash_functions.SHA256, version=1)
        except Exception as exc:
            err = "_validate_hash_function() failed with a valid hash function and version"
            self.fail(f"{err}: {exc}")

        with self.assertRaises(exceptions.HashFunction):
            _validators._validate_hash_function(hash_function="not123", version=1)

    def test_validate_fingerprint_pattern(self):
        """Test json fingerprint pattern validator.

        Verify that:
        - FingerprintPattern exception is not raised with a valid fingerprint
        - FingerprintPattern exception is raised with an invalid fingerprint
        """
        with self.assertRaises(exceptions.FingerprintPattern):
            _validators._validate_fingerprint_format(fingerprint="invalid fingerprint")

        input = json.dumps({"foo": "bar"})
        jfpv1_sha256 = create(input=input, hash_function=hash_functions.SHA256, version=1)
        jfpv1_sha384 = create(input=input, hash_function=hash_functions.SHA384, version=1)
        jfpv1_sha512 = create(input=input, hash_function=hash_functions.SHA512, version=1)

        try:
            _validators._validate_fingerprint_format(fingerprint=jfpv1_sha256)
        except Exception as exc:
            err = "_validate_fingerprint_format() failed with a valid json-sha256 fingerprint"
            self.fail(f"{err}: {exc}")

        try:
            _validators._validate_fingerprint_format(fingerprint=jfpv1_sha384)
        except Exception as exc:
            err = "_validate_fingerprint_format() failed with a valid json-sha384 fingerprint"
            self.fail(f"{err}: {exc}")

        try:
            _validators._validate_fingerprint_format(fingerprint=jfpv1_sha512)
        except Exception as exc:
            err = "_validate_fingerprint_format() failed with a valid json-sha512 fingerprint"
            self.fail(f"{err}: {exc}")

        fp = "jfpv1$exception_test"
        with self.assertRaises(exceptions.FingerprintPattern):
            _validators._validate_fingerprint_format(fingerprint=fp)


if __name__ == "__main__":
    unittest.main()



from typing import Dict, List

from ._create import create
from ._decode import decode


def _get_target_hashes(fingerprints: List[str]) -> List[Dict]:
    target_hashes = []
    for fingerprint in fingerprints:
        version, hash_function, _ = decode(fingerprint=fingerprint)
        element = {"version": version, "hash_function": hash_function}
        if element not in target_hashes:
            target_hashes.append(element)

    return target_hashes


def _create_input_fingerprints(input: str, target_hashes: List[Dict]) -> List[str]:
    """Create all necessary JSON fingerprint variations of the JSON data input."""
    input_fingerprints = []
    for element in target_hashes:
        fingerprint = create(input=input, hash_function=element["hash_function"], version=element["version"])
        input_fingerprints.append(fingerprint)
    return input_fingerprints


def find_matches(input: str, fingerprints: List[str], deduplicate: bool = False) -> List[str]:
    """Match raw json string input to a list of fingerprints.

    The fingerprint matching is executed as follows:
        1. Optional: deduplicate the target fingerprint list
        2. Decode the target fingerprint list to find all different JSON fingerprint variations (hash function, version)
        3. Create JSON fingerprints from the input with identical parameters (e.g., all hash function variations)
        4. Compare the input JSON fingerprint variations to the target list and find all matches

    If there is e.g. "sha256" and "sha512" JSON fingerprint variations in the target list of fingerprints,
    then a JSON fingerprint is created from the input JSON data with both the "sha256" and the "sha512" hash functions.

    Args:
        input (str):
            JSON input in string format.
        fingerprints (list of strings):
            A list of JSON fingerprints in string format.
        deduplicate (bool):
            If True, then deduplicate the fingerprint list before processing matches. False by default.

    Returns:
        list: A list of JSON fingerprint matches in string format.
    """
    if deduplicate:
        fingerprints = list(set(fingerprints))
    target_hashes = _get_target_hashes(fingerprints=fingerprints)
    input_fingerprints = _create_input_fingerprints(input=input, target_hashes=target_hashes)

    matches = []
    for fingerprint in fingerprints:
        if fingerprint in input_fingerprints:
            matches.append(fingerprint)

    return matches



from typing import Tuple

from ._validators import _validate_fingerprint_format


def decode(fingerprint: str) -> Tuple[int, str, str]:
    """Decode json fingerprints into version, hash function, and the hex digest of the hash value.

    Args:
        fingerprint (str):
            A valid JSON fingerprint (example: "jfpv1$sha256$2ecb0c919fcb06024f55380134da3bbaac3879f98adce89a8871706fe50dda03")

    Returns:
        A tuple containing the following items
        - int: JSON fingerprint version (example: 1)
        - str: hash function name (example: "sha256")
        - str: a hex digest of the secure hash value (example: "2ecb0c919fcb06024f55380134da3bbaac3879f98adce89a8871706fe50dda03")
    """
    _validate_fingerprint_format(fingerprint)
    elements = fingerprint.split("$")
    version = int(elements[0][4:])
    hash_function = elements[1]
    hash = elements[2]

    return version, hash_function, hash



from ._create import create
from ._decode import decode


def match(input: str, target_fingerprint: str) -> bool:
    """Match raw json string input to target fingerprint.

    Decodes the target fingerprint and creates a fingerprint from the input with identical parameters.
    If the target fingerprint uses, for example, the "sha256" hash function, then a JSON fingerprint
    with "sha256" is also generated prior to matching.

    Args:
        input (str):
            JSON input in string format.
        target_fingerprint (str):
            Target JSON fingerprint in string format.

    Returns:
        bool: True if the input JSON data matches with the target fingerprint, otherwise False.
    """
    version, hash_function, _ = decode(fingerprint=target_fingerprint)
    input_fingerprint = create(input=input, hash_function=hash_function, version=version)
    if input_fingerprint == target_fingerprint:
        return True
    return False
